//libraries needed for the code
#include <TimerFreeTone.h> //library used for instead of pitches because of timer error
#define TONE_PIN 3 // a pin to define the output source of the song
#include <Wire.h> //for the lcd
#include <LiquidCrystal_I2C.h> //Library to function the lcd

int trig = 5; //PWM pin
int echo = 6; //PWM pin
int pir = 7; //normal, digital pin 
int Buzzer = 3; //PWM pin
int redLED = 2; //normal, digital pin
int yellowLED = 4; //normal, digital pin
int redRGB = 9; //PWM pin
int greenRGB = 10; //PWM pin
int blueRGB = 11; //PWM pin
LiquidCrystal_I2C lcd(0x27, 16, 2); //lcd display address

void setup() { //set up the inputs, outputs and the LCD I2C
  pinMode(redLED, OUTPUT); 
  pinMode(yellowLED, OUTPUT);
  pinMode(Buzzer, OUTPUT);
  pinMode(pir, INPUT); 
  pinMode(redRGB, OUTPUT);
  pinMode(blueRGB, OUTPUT);
  pinMode(greenRGB, OUTPUT);
  pinMode(trig, OUTPUT);
  pinMode(echo, INPUT);
  lcd.init(); //initilazing lcd
  lcd.backlight(); //turn on lcd backlight
}

void setColour(int r, int g, int b) { //setting up the RGB led to do the following 

  //important and for Common Anode RGB LED ONLY!!
  r = 255-r;
  g = 255-g;
  b = 255-b;
   
  analogWrite(redRGB,r);
  analogWrite(greenRGB,g);
  analogWrite(blueRGB,b);
}

void siren (){ //the music for intro and outro of the program --> uses the pitches library
    int melody[] = { //notes or in other words 1st half of sheet music
      //Sirens 
    370 , 370, 262, 262, 370, 370, 262, 262, 370, 370, 262, 262, 370, 370, 262, 262, 370
    // f#, f#, c, c, f#, f#, c, c, f#, f#, c, c, f#, f#, c, c, f# -> Notes above 
  };
  
  // The note duration, 8 = 8th note, 4 = quarter note, etc.
  int durations[] = { //duration of each note or in other words 2nd half of sheet music
    4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 16, 16,
  };
  // the length of the arrays to use in the loop iteration
  int songLength = sizeof(melody)/sizeof(melody[0]);
    for (int thisNote = 0; thisNote < songLength; thisNote++){
      int duration = 1200/ durations[thisNote];
      TimerFreeTone(TONE_PIN, melody[thisNote], duration);
      int pause = duration * 0.1;
       // Play thisNote for duration.
      delay(pause);   
  }
}

void warning(){ //series of lights coded on the RGB LED
  setColour(204,0,0); //dark red
  delay(600);
  setColour(0,0,255); // blue
  delay(600);
  setColour(255,51,51); // light red
  delay(600);
  setColour(0,0,153); //dark blue
  delay(600); 
  setColour(255,128,0); //orange
  delay(600);
  setColour(0,0,255); // blue
  delay(600);
  setColour(255,108,92); //blush
  delay(600);
}

void loop() { //the main piece of code
  long duration, distance; //Distance sensor setup code
  digitalWrite(trig, LOW); //turn the 'ping' off
  delayMicroseconds(2); //delay for 2 milliseconds
  digitalWrite(trig, HIGH);  //turn the 'ping' on
  delayMicroseconds(10); //delay for 10 milliseconds, long enough for the ping to reach an object
  digitalWrite(trig, LOW); //turn the 'ping' off
  delayMicroseconds(2); //delay for 2 milliseconds

  duration = pulseIn(echo, HIGH); //turn echo pin on and listen for the ping
  distance = (duration/2)/74; //distance calculated in inches

  if ( digitalRead(pir) == 1){ //if the pir detects motion then
    digitalWrite(redLED, LOW); //turn off red led
    digitalWrite(yellowLED, HIGH); //turn on yellow led
    lcd.setCursor(0,0); //set the cursor on the lcd to the first row, first coloumn
    lcd.println("Motion detected!!"); //write the following on the LCD
    delay(1000); //pause for 1000 seconds
    lcd.clear(); //clear the lcd screen
  }

  if(digitalRead(pir) == 0){ //if the pir detects no motion then
    digitalWrite(yellowLED, LOW); //turn yellow led off
    digitalWrite(redLED, LOW); //turn red led off
    setColour(0,102,0); //use the setColour function from before and make the RGB led green
    lcd.setCursor(0,0); //set the cursor on the lcd to the first row, first coloumn
    lcd.print("All Clear       ");//write the following on the LCD
    delay(1000); //pause for 3000 seconds
    lcd.clear(); //clear the lcd screen
  }

  if (distance >= 41){ //if the distance sensor is greater than or equal to 41 inches/ 1 meter, then:
    lcd.setCursor(0,0); //set the cursor on the lcd to the first row, first coloumn
    lcd.print("Distance:            "); //write the following on the LCD
    lcd.setCursor(0,1); //set the cursor on the lcd to the second row, first coloumn
    lcd.print(distance); //print the distance calculated by the distance sensor onto the lcd
    lcd.setCursor(2,1); //set the cursor on the lcd to the second row, second coloumn
    lcd.print(" inches              "); //write the following on the LCD
    delay(1000); //pause for 3000 seconds
    lcd.clear(); //clear the lcd screen
    
  }
  if (distance <= 40){ //about 1 meter
    digitalWrite(yellowLED, LOW); //turn off yellow led
    digitalWrite(redLED, HIGH); //turn on red led
    setColour(0,0,0); // turn off rgb led
    lcd.setCursor(0,0); //set the cursor on the lcd to the first row, first coloumn
    lcd.print("Intruder alert! "); //write the following on the LCD
    lcd.setCursor(0,1); //set the cursor on the lcd to the second row, first coloumn
    lcd.print(distance); //print the distance calculated by the distance sensor onto the lcd
    lcd.setCursor(2,1); //set the cursor on the lcd to the second row, second coloumn
    lcd.print(" inches                    "); //write the following on the LCD
    siren(); //the void code from above, used to immitate a siren
    warning(); //void code from above with different colours coded, immitating police siren lights
    siren(); //the void code from above, used to immitate a siren
    delay(1000); //pause for 3000 seconds
    lcd.clear(); //clear the lcd screen
  }
}
